@@ -11,12 +11,10 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-#include "autocompleter.h"
-#include "textbuffer.h"
-#include "util.h"
+#include "t3widget/autocompleter.h"
 
-namespace t3_widget {
+namespace t3widget {
 
 autocompleter_t::~autocompleter_t() {}
 
-};  // namespace
+}  // namespace t3widget
