@@ -14,32 +14,28 @@
 #ifndef T3_WIDGET_CLIPBOARD_H
 #define T3_WIDGET_CLIPBOARD_H
 
+#include <memory>
 #include <string>
-#include <t3widget/ptr.h>
 #include <t3widget/widget_api.h>
+#include <utility>
 
-#define WITH_CLIPBOARD_LOCK(code)             \
-  {                                           \
-    t3_widget::ensure_clipboard_lock_t _lock; \
-    code                                      \
-  }
+namespace t3widget {
 
-namespace t3_widget {
+T3_WIDGET_API std::shared_ptr<std::string> get_clipboard();
+T3_WIDGET_API std::shared_ptr<std::string> get_primary();
 
-T3_WIDGET_API linked_ptr<std::string>::t get_clipboard();
-T3_WIDGET_API linked_ptr<std::string>::t get_primary();
-
-T3_WIDGET_API void set_clipboard(std::string *str);
-T3_WIDGET_API void set_primary(std::string *str);
+T3_WIDGET_API void set_clipboard(std::unique_ptr<std::string> str);
+T3_WIDGET_API void set_primary(std::unique_ptr<std::string> str);
 T3_WIDGET_API void release_selections();
 T3_WIDGET_API void lock_clipboard();
 T3_WIDGET_API void unlock_clipboard();
 
+/** A simple class to use RAII to ensure that the clipboard lock is held and released if and when
+    necessary. */
 class T3_WIDGET_API ensure_clipboard_lock_t {
  public:
   ensure_clipboard_lock_t() { lock_clipboard(); }
   ~ensure_clipboard_lock_t() { unlock_clipboard(); }
 };
-
-};  // namespace
+}  // namespace t3widget
 #endif
