@@ -14,24 +14,31 @@
 #ifndef T3_WIDGET_GOTODIALOG_H
 #define T3_WIDGET_GOTODIALOG_H
 
+#include <functional>
 #include <t3widget/dialogs/dialog.h>
-#include <t3widget/widgets/textfield.h>
+#include <t3widget/util.h>
 
-namespace t3_widget {
+#include <t3widget/signals.h>
+#include <t3widget/widget_api.h>
+
+namespace t3widget {
 
 class T3_WIDGET_API goto_dialog_t : public dialog_t {
  private:
-  text_field_t *number_line;
+  struct T3_WIDGET_LOCAL implementation_t;
+
+  single_alloc_pimpl_t<implementation_t> impl;
 
   void ok_activate();
 
  public:
   goto_dialog_t();
+  ~goto_dialog_t() override;
   bool set_size(optint height, optint width) override;
   void reset();
 
-  T3_WIDGET_SIGNAL(activate, void, int);
+  connection_t connect_activate(std::function<void(text_pos_t)> cb);
 };
 
-};  // namespace
+}  // namespace t3widget
 #endif
