@@ -18,20 +18,22 @@
 #include <t3widget/dialogs/dialog.h>
 #include <t3widget/widgets/textfield.h>
 
-namespace t3_widget {
+namespace t3widget {
 
 class T3_WIDGET_API insert_char_dialog_t : public dialog_t {
  private:
-  text_field_t *description_line;
-  key_t interpret_key(const std::string *descr);
+  struct T3_WIDGET_LOCAL implementation_t;
+  single_alloc_pimpl_t<implementation_t> impl;
+  key_t interpret_key(const std::string &descr);
 
  public:
   insert_char_dialog_t();
+  ~insert_char_dialog_t() override;
   bool set_size(optint height, optint width) override;
   void reset();
 
   void ok_activate();
 };
 
-};  // namespace
+}  // namespace t3widget
 #endif
