@@ -16,29 +16,37 @@
 
 #include <t3widget/dialogs/dialogbase.h>
 
-namespace t3_widget {
+#include <t3widget/interfaces.h>
+#include <t3widget/key.h>
+#include <t3widget/mouse.h>
+#include <t3widget/util.h>
+#include <t3widget/widget_api.h>
+
+namespace t3widget {
 
 /** Base class for dialogs. */
 class T3_WIDGET_API popup_t : public dialog_base_t, public mouse_target_t {
  private:
   friend class dialog_t;
 
-  bool draw;
-  bool shown;
+  struct T3_WIDGET_LOCAL implementation_t;
+
+  single_alloc_pimpl_t<implementation_t> impl;
 
  protected:
   popup_t(int height, int width, bool shadow = true, bool _draw = true);
 
  public:
+  ~popup_t() override;
   bool process_key(key_t key) override;
   void update_contents() override;
   void show() override;
   void hide() override;
   bool process_mouse_event(mouse_event_t event) override;
 
-  bool is_shown();
+  bool is_shown() const;
 };
 
-}  // namespace
+}  // namespace t3widget
 
 #endif
