@@ -17,7 +17,11 @@
 #include <cstdint>
 #include <cstring>
 #include <memory>
+#ifdef PCRE_COMPAT
+#include "t3widget/pcre_compat.h"
+#else
 #include <pcre2.h>
+#endif
 #include <string>
 #include <unicase.h>
 
@@ -192,7 +196,6 @@ bool plain_finder_t::match(const std::string &haystack, find_result_t *result, b
   }
 
   size_t c_size;
-  const char *c;
 
   text_pos_t start = std::max<text_pos_t>(0, result->start.pos);
   if (static_cast<size_t>(start) > haystack.size()) {
@@ -221,7 +224,6 @@ bool plain_finder_t::match(const std::string &haystack, find_result_t *result, b
         char *c_data = reinterpret_cast<char *>(
             u8_casefold(reinterpret_cast<const uint8_t *>(substr.data()), substr.size(), nullptr,
                         nullptr, reinterpret_cast<uint8_t *>(folded_.get()), &c_size));
-        c = c_data;
         if (c_data != folded_.get()) {
           // Previous value of folded will be automatically deleted.
           folded_.reset(c_data);
@@ -330,7 +332,7 @@ bool regex_finder_t::set_needle(const std::string &needle, std::string *error_me
     pcre_flags |= PCRE2_CASELESS;
   }
 
-  regex_.reset(pcre2_compile_8(reinterpret_cast<PCRE2_SPTR8>(pattern.data()), pattern.size(),
+  regex_.reset(pcre2_compile_8(reinterpret_cast<PCRE2_SPTR8>(pattern.c_str()), pattern.size(),
                                pcre_flags, &error_code, &error_offset, nullptr));
   if (regex_ == nullptr) {
     char buffer[256];
