@@ -12,10 +12,10 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "key_binding.h"
+#include "t3widget/key_binding.h"
 
-namespace t3_widget {
+namespace t3widget {
 
 key_bindings_base_t::~key_bindings_base_t() {}
 
-}  // namespace
+}  // namespace t3widget
