@@ -23,9 +23,9 @@
 #endif
 
 #include <stdio.h>
+#include <t3widget/widget_api.h>
 
-#include "widget_api.h"
-namespace t3_widget {
+namespace t3widget {
 
 #ifdef _T3_WIDGET_DEBUG
 
@@ -39,12 +39,15 @@ T3_WIDGET_API void lprintf(const char *fmt, ...)
     ;
 T3_WIDGET_API void ldumpstr(const char *str, int length);
 T3_WIDGET_API void logkeyseq(const char *keys);
+
 #else
+
 #define init_log()
 #define lprintf(fmt, ...)
 #define ldumpstr(str, length)
 #define logkeyseq(keys)
+
 #endif
 
-};  // namespace
+}  // namespace t3widget
 #endif
