@@ -14,13 +14,17 @@
 #ifndef T3_WIDGET_STRINGMATCHER_H
 #define T3_WIDGET_STRINGMATCHER_H
 
+#ifndef _T3_WIDGET_INTERNAL
+#error This header file is for internal use _only_!!
+#endif
+
 #include <memory>
 #include <string>
 #include <t3widget/util.h>
 #include <t3widget/widget_api.h>
 #include <vector>
 
-namespace t3_widget {
+namespace t3widget {
 
 class T3_WIDGET_LOCAL string_matcher_t {
  private:
@@ -30,15 +34,12 @@ class T3_WIDGET_LOCAL string_matcher_t {
   void init();
 
  public:
-  string_matcher_t(const std::string &_needle);
-  string_matcher_t(char *_needle, size_t _needle_size);
+  string_matcher_t(string_view _needle);
   virtual ~string_matcher_t();
   void reset();
-  int next_char(const std::string *c);
-  int previous_char(const std::string *c);
-  int next_char(const char *c, size_t c_size);
-  int previous_char(const char *c, size_t c_size);
+  int next_char(string_view c);
+  int previous_char(string_view c);
 };
 
-};  // namespace
+}  // namespace t3widget
 #endif
