@@ -134,6 +134,18 @@ struct edit_window_t::implementation_t {
       repaint_max = std::numeric_limits<text_pos_t>::max(); /**< Last line to repaint. */
 };
 
+struct edit_window_t::behavior_parameters_t::implementation_t {
+  text_coordinate_t top_left{0, 0};
+  text_pos_t last_set_pos = 0;
+  int tabsize = 8;
+  int ins_mode = 0;
+  wrap_type_t wrap_type = wrap_type_t::NONE;
+  bool tab_spaces = false;
+  bool auto_indent = true;
+  bool indent_aware_home = true;
+  bool show_tabs = false;
+};
+
 void edit_window_t::init(bool _init) {
   if (_init) {
     /* Construct these from t3widget::init, such that the locale is set correctly and
@@ -153,11 +165,12 @@ void edit_window_t::init(bool _init) {
     global_find_dialog = nullptr;
     delete replace_buttons;
     replace_buttons = nullptr;
+    delete right_click_menu;
+    right_click_menu = nullptr;
   }
 }
 
-edit_window_t::edit_window_t(text_buffer_t *_text, const view_parameters_t *params)
-    : impl(new implementation_t()), text(nullptr) {
+void edit_window_t::init_instance() {
   /* Register the unbacked window for mouse events, such that we can get focus
      if the bottom line is clicked. */
   init_unbacked_window(11, 11, true);
@@ -183,8 +196,6 @@ edit_window_t::edit_window_t(text_buffer_t *_text, const view_parameters_t *para
   impl->scrollbar->connect_clicked(bind_front(&edit_window_t::scrollbar_clicked, this));
   impl->scrollbar->connect_dragged(bind_front(&edit_window_t::scrollbar_dragged, this));
 
-  set_text(_text == nullptr ? new text_buffer_t() : _text, params);
-
   impl->screen_pos = 0;
   impl->focus = false;
 
@@ -192,12 +203,36 @@ edit_window_t::edit_window_t(text_buffer_t *_text, const view_parameters_t *para
   impl->autocomplete_panel->connect_activate([this] { autocomplete_activated(); });
 }
 
+edit_window_t::edit_window_t(text_buffer_t *_text, const view_parameters_t *params)
+    : widget_t(impl_alloc<implementation_t>(0)), impl(new_impl<implementation_t>()), text(nullptr) {
+  init_instance();
+  set_text(_text == nullptr ? new text_buffer_t() : _text, params);
+}
+
+edit_window_t::edit_window_t(text_buffer_t *_text, const behavior_parameters_t *params)
+    : widget_t(impl_alloc<implementation_t>(0)), impl(new_impl<implementation_t>()), text(nullptr) {
+  init_instance();
+  set_text(_text == nullptr ? new text_buffer_t() : _text, params);
+}
+
 edit_window_t::~edit_window_t() { delete impl->wrap_info; }
 
 void edit_window_t::set_text(text_buffer_t *_text, const view_parameters_t *params) {
   if (text == _text) {
     return;
   }
+  if (params != nullptr) {
+    behavior_parameters_t new_params(*params);
+    set_text(_text, &new_params);
+  } else {
+    set_text(_text, static_cast<behavior_parameters_t *>(nullptr));
+  }
+}
+
+void edit_window_t::set_text(text_buffer_t *_text, const behavior_parameters_t *params) {
+  if (text == _text) {
+    return;
+  }
 
   text = _text;
   if (params != nullptr) {
@@ -219,7 +254,12 @@ void edit_window_t::set_text(text_buffer_t *_text, const view_parameters_t *para
 
 bool edit_window_t::set_size(optint height, optint width) {
   bool result = true;
-  // FIXME: these int's are optional!!! Take that into account below!
+  if (!height.is_valid()) {
+    height = window.get_height();
+  }
+  if (!width.is_valid()) {
+    width = window.get_width();
+  }
 
   if (width.value() != window.get_width() || height.value() > window.get_height()) {
     update_repaint_lines(0, std::numeric_limits<text_pos_t>::max());
@@ -351,7 +391,7 @@ void edit_window_t::repaint_screen() {
         info.selection_end = -1;
       }
 
-      info.cursor = impl->focus && impl->top_left.line + i == cursor.line ? cursor.pos : -1;
+      info.cursor = impl->top_left.line + i == cursor.line ? cursor.pos : -1;
       impl->edit_window.set_paint(i, 0);
       impl->edit_window.clrtoeol();
       text->paint_line(&impl->edit_window, impl->top_left.line + i, info);
@@ -378,7 +418,7 @@ void edit_window_t::repaint_screen() {
         info.selection_end = -1;
       }
 
-      info.cursor = impl->focus && draw_line.line == cursor.line ? cursor.pos : -1;
+      info.cursor = draw_line.line == cursor.line ? cursor.pos : -1;
       impl->edit_window.set_paint(i, 0);
       impl->edit_window.clrtoeol();
       impl->wrap_info->paint_line(&impl->edit_window, draw_line, info);
@@ -783,17 +823,19 @@ void edit_window_t::find_activated(std::shared_ptr<finder_t> _finder, find_actio
       break;
     case find_action_t::REPLACE_ALL: {
       int replacements;
-      text_coordinate_t start(0, 0);
+      text_coordinate_t start(0, -1);
       text_coordinate_t eof(std::numeric_limits<text_pos_t>::max(),
                             std::numeric_limits<text_pos_t>::max());
 
       for (replacements = 0; text->find_limited(local_finder, start, eof, &result);
            replacements++) {
+        lprintf("Find result: %ld %ld\n", result.start.line, result.start.pos);
         if (replacements == 0) {
           text->start_undo_block();
         }
         text->replace(*local_finder, result);
         start = text->get_cursor();
+        lprintf("New start: %ld %ld\n", start.line, start.pos);
       }
 
       if (replacements == 0) {
@@ -1010,6 +1052,29 @@ bool edit_window_t::process_key(key_t key) {
       impl->last_set_pos = impl->screen_pos;
       break;
     }
+    case EKEY_BS | EKEY_CTRL:
+      if (text->get_selection_mode() == selection_mode_t::NONE) {
+        const text_coordinate_t cursor = text->get_cursor();
+        if (cursor.pos <= text->get_line_size(cursor.line)) {
+          if (cursor.pos != 0) {
+            text->backspace_word();
+            update_repaint_lines(cursor.line);
+          } else if (cursor.line != 0) {
+            text->merge(true);
+            update_repaint_lines(cursor.line);
+          }
+        } else {
+          ASSERT(false);
+        }
+        ensure_cursor_on_screen();
+        impl->last_set_pos = impl->screen_pos;
+      } else {
+        delete_selection();
+      }
+      if (impl->autocomplete_panel->is_shown()) {
+        activate_autocomplete(false);
+      }
+      break;
     case EKEY_BS:
       if (text->get_selection_mode() == selection_mode_t::NONE) {
         const text_coordinate_t cursor = text->get_cursor();
@@ -1041,36 +1106,10 @@ bool edit_window_t::process_key(key_t key) {
       break;
 
     case EKEY_SHIFT | '\t':
-      if (text->get_selection_mode() != selection_mode_t::NONE &&
-          text->get_selection_start().line != text->get_selection_end().line) {
-        unindent_selection();
-      } else {
-        text->unindent_line(impl->tabsize);
-        ensure_cursor_on_screen();
-        impl->last_set_pos = impl->screen_pos;
-        text_pos_t cursor_line = text->get_cursor().line;
-        update_repaint_lines(cursor_line);
-      }
+      unindent_selection();
       break;
     case '\t':
-      if (text->get_selection_mode() != selection_mode_t::NONE &&
-          text->get_selection_start().line != text->get_selection_end().line) {
-        indent_selection();
-      } else {
-        std::string space;
-        if (text->get_selection_mode() != selection_mode_t::NONE) {
-          delete_selection();
-        }
-
-        if (impl->tab_spaces) {
-          space.append(impl->tabsize - (impl->screen_pos % impl->tabsize), ' ');
-        } else {
-          space.append(1, '\t');
-        }
-        text->insert_block(space);
-        ensure_cursor_on_screen();
-        impl->last_set_pos = impl->screen_pos;
-      }
+      indent_selection();
       break;
     case EKEY_PASTE_START:
       if (!impl->pasting_text) {
@@ -1188,7 +1227,7 @@ void edit_window_t::update_contents() {
      - cursor-only movements mostly don't require entire redraws [well, that depends:
         for matching brace/parenthesis it may require more than a single line update]
   */
-  if (!impl->focus && !reset_redraw()) {
+  if (!reset_redraw()) {
     return;
   }
 
@@ -1329,6 +1368,8 @@ void edit_window_t::right_click_menu_activated(int action) {
     case ACTION_PASTE_SELECTION:
       paste(false);
       break;
+    default:
+      break;
   }
 }
 
@@ -1344,17 +1385,43 @@ void edit_window_t::insert_special() {
 }
 
 void edit_window_t::indent_selection() {
-  text->indent_selection(impl->tabsize, impl->tab_spaces);
-  ensure_cursor_on_screen();
-  impl->last_set_pos = impl->screen_pos;
-  update_repaint_lines(text->get_selection_start().line, text->get_selection_end().line);
+  if (text->get_selection_mode() != selection_mode_t::NONE &&
+      text->get_selection_start().line != text->get_selection_end().line) {
+    text->indent_selection(impl->tabsize, impl->tab_spaces);
+    ensure_cursor_on_screen();
+    impl->last_set_pos = impl->screen_pos;
+    update_repaint_lines(text->get_selection_start().line, text->get_selection_end().line);
+  } else {
+    std::string space;
+    if (text->get_selection_mode() != selection_mode_t::NONE) {
+      delete_selection();
+    }
+
+    if (impl->tab_spaces) {
+      space.append(impl->tabsize - (impl->screen_pos % impl->tabsize), ' ');
+    } else {
+      space.append(1, '\t');
+    }
+    text->insert_block(space);
+    ensure_cursor_on_screen();
+    impl->last_set_pos = impl->screen_pos;
+  }
 }
 
 void edit_window_t::unindent_selection() {
-  text->unindent_selection(impl->tabsize);
-  ensure_cursor_on_screen();
-  impl->last_set_pos = impl->screen_pos;
-  update_repaint_lines(text->get_selection_start().line, text->get_selection_end().line);
+  if (text->get_selection_mode() != selection_mode_t::NONE &&
+      text->get_selection_start().line != text->get_selection_end().line) {
+    text->unindent_selection(impl->tabsize);
+    ensure_cursor_on_screen();
+    impl->last_set_pos = impl->screen_pos;
+    update_repaint_lines(text->get_selection_start().line, text->get_selection_end().line);
+  } else {
+    text->unindent_line(impl->tabsize);
+    ensure_cursor_on_screen();
+    impl->last_set_pos = impl->screen_pos;
+    text_pos_t cursor_line = text->get_cursor().line;
+    update_repaint_lines(cursor_line);
+  }
 }
 
 void edit_window_t::goto_line() {
@@ -1594,6 +1661,15 @@ void edit_window_t::save_view_parameters(view_parameters_t *params) {
   *params = view_parameters_t(this);
 }
 
+std::unique_ptr<edit_window_t::behavior_parameters_t> edit_window_t::save_behavior_parameters() {
+  // This can't use make_unique, as the constructor is private and only this class is a friend.
+  return wrap_unique(new behavior_parameters_t(*this));
+}
+
+void edit_window_t::save_behavior_parameters(behavior_parameters_t *params) {
+  *params = behavior_parameters_t(*this);
+}
+
 void edit_window_t::draw_info_window() {}
 
 void edit_window_t::set_autocompleter(autocompleter_t *_autocompleter) {
@@ -1895,6 +1971,102 @@ void edit_window_t::view_parameters_t::set_indent_aware_home(bool _indent_aware_
 }
 void edit_window_t::view_parameters_t::set_show_tabs(bool _show_tabs) { show_tabs = _show_tabs; }
 
+//====================== behavior_parameters_t ========================
+
+edit_window_t::behavior_parameters_t::behavior_parameters_t(const edit_window_t &view)
+    : impl(new implementation_t) {
+  impl->top_left = view.impl->top_left;
+  impl->wrap_type = view.impl->wrap_type;
+  if (impl->wrap_type != wrap_type_t::NONE) {
+    impl->top_left.pos =
+        view.impl->wrap_info->calculate_line_pos(impl->top_left.line, 0, impl->top_left.pos);
+  }
+  impl->tabsize = view.impl->tabsize;
+  impl->tab_spaces = view.impl->tab_spaces;
+  impl->ins_mode = view.impl->ins_mode;
+  impl->last_set_pos = view.impl->last_set_pos;
+  impl->auto_indent = view.impl->auto_indent;
+  impl->indent_aware_home = view.impl->indent_aware_home;
+  impl->show_tabs = view.impl->show_tabs;
+}
+
+edit_window_t::behavior_parameters_t::behavior_parameters_t() : impl(new implementation_t) {}
+
+edit_window_t::behavior_parameters_t::behavior_parameters_t(
+    const edit_window_t::view_parameters_t &params)
+    : impl(new implementation_t) {
+  impl->top_left = params.top_left;
+  impl->wrap_type = params.wrap_type;
+  impl->tabsize = params.tabsize;
+  impl->tab_spaces = params.tab_spaces;
+  impl->ins_mode = params.ins_mode;
+  impl->last_set_pos = params.last_set_pos;
+  impl->auto_indent = params.auto_indent;
+  impl->indent_aware_home = params.indent_aware_home;
+  impl->show_tabs = params.show_tabs;
+}
+
+/* Destructor must be defined in the .cc file, because in the .h file the size of implementation_t
+   is undefined. */
+edit_window_t::behavior_parameters_t::~behavior_parameters_t() {}
+
+edit_window_t::behavior_parameters_t &edit_window_t::behavior_parameters_t::operator=(
+    const behavior_parameters_t &other) {
+  *impl = *other.impl;
+  return *this;
+}
+
+void edit_window_t::behavior_parameters_t::apply_parameters(edit_window_t *view) const {
+  view->impl->top_left = impl->top_left;
+  view->impl->tabsize = impl->tabsize;
+  view->set_wrap(impl->wrap_type);
+  /* view->set_wrap will make sure that view->wrap_info is nullptr if
+     wrap_type != NONE. */
+  if (view->impl->wrap_info != nullptr) {
+    view->impl->wrap_info->set_text_buffer(view->text);
+    view->impl->top_left.pos = view->impl->wrap_info->find_line(impl->top_left);
+  }
+  // the calling function will call ensure_cursor_on_screen
+  view->impl->tab_spaces = impl->tab_spaces;
+  view->impl->ins_mode = impl->ins_mode;
+  view->impl->last_set_pos = impl->last_set_pos;
+  view->impl->auto_indent = impl->auto_indent;
+  view->impl->indent_aware_home = impl->indent_aware_home;
+  view->impl->show_tabs = impl->show_tabs;
+}
+
+void edit_window_t::behavior_parameters_t::set_tabsize(int _tabsize) { impl->tabsize = _tabsize; }
+void edit_window_t::behavior_parameters_t::set_wrap(wrap_type_t _wrap_type) {
+  impl->wrap_type = _wrap_type;
+}
+void edit_window_t::behavior_parameters_t::set_tab_spaces(bool _tab_spaces) {
+  impl->tab_spaces = _tab_spaces;
+}
+void edit_window_t::behavior_parameters_t::set_auto_indent(bool _auto_indent) {
+  impl->auto_indent = _auto_indent;
+}
+void edit_window_t::behavior_parameters_t::set_indent_aware_home(bool _indent_aware_home) {
+  impl->indent_aware_home = _indent_aware_home;
+}
+void edit_window_t::behavior_parameters_t::set_show_tabs(bool _show_tabs) {
+  impl->show_tabs = _show_tabs;
+}
+void edit_window_t::behavior_parameters_t::set_top_left(text_coordinate_t pos) {
+  impl->top_left = pos;
+}
+
+int edit_window_t::behavior_parameters_t::get_tabsize() const { return impl->tabsize; }
+wrap_type_t edit_window_t::behavior_parameters_t::get_wrap_type() const { return impl->wrap_type; }
+bool edit_window_t::behavior_parameters_t::get_tab_spaces() const { return impl->tab_spaces; }
+bool edit_window_t::behavior_parameters_t::get_auto_indent() const { return impl->auto_indent; }
+bool edit_window_t::behavior_parameters_t::get_indent_aware_home() const {
+  return impl->indent_aware_home;
+}
+bool edit_window_t::behavior_parameters_t::get_show_tabs() const { return impl->show_tabs; }
+text_coordinate_t edit_window_t::behavior_parameters_t::get_top_left() const {
+  return impl->top_left;
+}
+
 //====================== autocomplete_panel_t ========================
 
 edit_window_t::autocomplete_panel_t::autocomplete_panel_t(edit_window_t *parent)
