@@ -16,20 +16,28 @@
 
 #include <t3widget/widgets/widget.h>
 
-namespace t3_widget {
+#include <t3widget/interfaces.h>
+#include <t3widget/key.h>
+#include <t3widget/util.h>
+#include <t3widget/widget_api.h>
+
+namespace t3widget {
 
 class T3_WIDGET_API separator_t : public widget_t {
  private:
-  bool horizontal;
+  struct T3_WIDGET_LOCAL implementation_t;
+
+  single_alloc_pimpl_t<implementation_t> impl;
 
  public:
-  separator_t(bool _horizontal = true);
+  separator_t(bool horizontal = true);
+  ~separator_t() override;
   bool process_key(key_t key) override;
   bool set_size(optint height, optint width) override;
   void update_contents() override;
   void set_focus(focus_t focus) override;
-  bool accepts_focus() override;
+  bool accepts_focus() const override;
 };
 
-};  // namespace
+}  // namespace t3widget
 #endif
